/*
 * tulklut.c
 *
 * tulklut main console program
 *
 * Copyright (C) 2003-2004 Richard "riri_le_mechant" GILL <richard@nasgaia.org>
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <tulklut.h>

/*
#ifndef _
#define _(s)	s
#endif

#ifndef LOCALEDIR
#define LOCALEDIR "/usr/share/locale"
#endif
*/

static void usage(FILE *stream, int exitcode);
static int get_oneline(FILE *stream, char **buffer);

/*
 * tulklut usage:
 * tulklut { [string1 [string2] [...]] | <stdin> }
 *
 * If strings don't have any special shell meaning, they can be typed as is,
 * they should be quoted otherwise.
 * If no string is passed, it's assumed the strings come from standard input.
 */

int main(int argc, char **argv)
{
/*	setlocale (LC_CTYPE, "");
	setlocale (LC_MESSAGES, "");

	bindtextdomain (PACKAGE, LOCALEDIR);
	textdomain (PACKAGE);
*/
	if (argc > 1 && (! strcmp(argv[1], "-h") || ! strcmp(argv[1], "--help")))
	{
		if (argc == 2)
			usage(stdout, 0);
		else
			usage(stderr, 1);
	}
	/* FIXME: here we could check if user wants an interactive session,
		in case we'd use readline to propose a better prompt, etc..
		readline should be checked in configure of course
	*/

	if (argc > 1)
	{
		/* operate on each command line parameters */
		int i;
		for (i = 1; i < argc; i ++)
			printf("%s ", tulklut_it(argv[i], strlen(argv[i])));
		printf("\n");
	}
	else
	{
		/* operate from standard input */
		FILE *input = stdin;
		char *buffer;

		while (get_oneline(input, &buffer) > 0)
		{
			printf("%s\n", tulklut_it(buffer, strlen(buffer)));

		}

	}
	return 0;
}

static void usage(FILE *stream, int exitcode)
{
	fprintf(stream,
"%s, a Zorglang tranlator. Version %s\n"
"Copyright 2004 by Richard GILL\n"
"  usage: %s [flags] | [strings]\n"
"  with a list of strings to translate or one of the following flags:\n"
"  -h --help \n"
"If no flag, nor strings are passed to the command line, strings to convert\n"
"will be read from standard input\n\n"
"... Richard n'est pas mchant !\n\n",
		PACKAGE, VERSION, PACKAGE);
	if (exitcode != 0)
		exit(exitcode);
}

static int get_oneline(FILE *stream, char **buffer)
{
	int c;
	size_t pos = 0;
	size_t size = 4096;
	*buffer = (char *) malloc(size * sizeof(char));

	do
	{
		if (pos == (size - 1))
		{
			size *= 2;
			*buffer = (char *) realloc(*buffer, size * sizeof(char));
		}
		c = fgetc(stream);
		if (c == '\n' || c == EOF)
		{
			(*buffer)[pos] = '\0';
			break;
		}
		(*buffer)[pos] = c;
		++ pos;
	} while (1);
	return strlen(*buffer);
}
